{****************************************************}
{CodeSuckerList.p																			}
{}
{CodeSucker unit that contains routines pertaining to the list manager}
{}
{Written using Think Pascal v3.0}
{Requires at least System 6 to run}
{}
{ 1991  Mike van Kleef	-	All rights Reserved}
{----------------------------------------------------}
{Address (mail & network) until September 1992 : }
{	42 Melford Road, London,  E11-4PS, England}
{	vankleef@uk.ac.qmc.dcs   [Queen Mary College, London] }
{Address after September 1992 :}
{	Flat 5, 4 St. Quintin Ave., London, W10-6NU, England}
{----------------------------------------------------}
{CodeSucker is distributed in the hope that it will be useful , but WITHOUT ANY WARRANTY}
{Absolutely no-one on Earth accepts responsibility for the consequences of using this program}
{Everyone is hereby granted permission to delete, copy, modify & redistribute CodeSucker}
{}
{Last Change 12/12/91}
{****************************************************}

unit CodeSuckerList;

interface

	uses
		GlobalRoutines;

	const
		UnknownRSRC = ' 	';	{Used to indicate an unknown resoure (It's one that's very unlikely to exist)}

	function BuildList (theWindow: WindowPtr; theRect: Rect; columns: Integer; flags: Integer): ListHandle;
	procedure ChangeTypeList (var theList: ListHandle);
	procedure ChangeRInfoList (rType: ResType; var theList: ListHandle);
	function GetSelectedItem (var start: Integer; var theCell: Point; theList: ListHandle): Boolean;
	function GetSelectedRsrcItem (theList: ListHandle): ResType;
	function GetListArea (theList: ListHandle): Integer;

implementation

	type
		ResTypePtr = ^ResType;
		Str255Ptr = ^Str255;

{-----------------------------------------}
{Returns the resource ID number at position'which' in the given list of resource ID's}
	function GetRsrcIDListItem (theList: ListHandle; which: Integer): LongInt;
		const
			WhatSize = 7;	{Number of characters to get from the List's cell data}
		var
			dataPtr: Str255Ptr;	{Pointer to resource Type}
			resultSize: Integer;	{Size of result of getting data from cell}
			theCell: Point;			{cell from which to get data}
			columns: Integer;		{Number of columns in list}
	begin
		dataPtr := Str255Ptr(NewPtr(WhatSize));
		resultSize := WhatSize;
		columns := LoWord(LongInt(theList^^.userHandle));
		SetPt(theCell, (which + 1) mod columns, (which - 1) div columns);
		LGetCell(Ptr(dataPtr), resultSize, theCell, theList);
		IntPtr(dataPtr)^ := Band(IntPtr(dataPtr)^, $00FF) + BSL(WhatSize - 1, 8);	{Set string length byte}
		while dataPtr^[1] = ' ' do
			Delete(dataPtr^, 1, 1);	{Strip leading spaces from number to enable proper conversion}
		StringToNum(dataPtr^, GetRsrcIDListItem);
		DisPosPtr(Ptr(dataPtr));
	end;

{-----------------------------------------}
{Returns the resource Type at position'which' in the list}
	function GetTypeListItem (theList: ListHandle; which: Integer): LongInt;
		var
			dataPtr: ResTypePtr;	{Pointer to resource Type}
			resultSize: Integer;	{Size of result of getting data from cell}
			theCell: Point;			{cell from which to get data}
			columns: Integer;		{Number of columns in list}
	begin
		dataPtr := ResTypePtr(NewPtr(SizeOf(ResType)));
		resultSize := SizeOf(ResType);
		columns := LoWord(LongInt(theList^^.userHandle));
		SetPt(theCell, (which + 1) mod columns, (which - 1) div columns);
		LGetCell(Ptr(dataPtr), resultSize, theCell, theList);
		GetTypeListItem := LongPtr(dataPtr)^;
		DisPosPtr(Ptr(dataPtr));
	end;

{-----------------------------------------}
{Routine to exchange the contents of two cells.}
{Fairly generic routine that will copy cell data up to 30 chars long}
	procedure ExchangeCellContents (theList: ListHandle; this, that: Integer);
		const
			maxContent = 30;	{Max datalength to copy}
		var
			thisCell: Point;		{This cell}
			thatCell: Point;		{That cell}
			columns: Integer;	{Number of columns in list}
			thisMuch: Integer;	{This much data}
			thatMuch: Integer;	{That much data}
			thisPtr: Ptr;		{Pointer to this data}
			thatPtr: Ptr;		{Pointer to that data}
	begin
		columns := LoWord(LongInt(theList^^.userHandle));

		SetPt(thisCell, (this + 1) mod columns, (this - 1) div columns);
		thisMuch := maxContent;
		thisPtr := NewPtr(thisMuch);
		LGetCell(thisPtr, thisMuch, thisCell, theList);

		SetPt(thatCell, (that + 1) mod columns, (that - 1) div columns);
		thatMuch := maxContent;
		thatPtr := NewPtr(thatMuch);
		LGetCell(thatPtr, thatMuch, thatCell, theList);

		LSetCell(thisPtr, thisMuch, thatCell, theList);
		LSetCell(thatPtr, thatMuch, thisCell, theList);

		DisposPtr(thisPtr);
		DisposPtr(thatPtr);
	end;

{-----------------------------------------}
{Sorting routine using QuickSort sorting algorithm to alphabetically sort the types list}
{Need a sorting routine, because the resource Types list contains multiple columns, so}
{it would be difficult to sort the list whilst building it up.}
{what=True then ResType else String type.}
	procedure QuickSortTypesList (theList: ListHandle; first, last: Integer; what: Boolean);
		var
			privIndex: Integer;
{------}
{Used to partition the list for the QuickSort routine to operate correctly}
{Passed in as FUNCTION is the routine that is used to sort either the Resource type list or the resource ID list}
		procedure PartitionGeneric (theList: ListHandle; first, last: Integer; var privIndex: Integer; function GetCellData (theList: ListHandle; which: Integer): LongInt);
			var
				pivot: LongInt;
				up: Integer;
				down: Integer;
		begin
			pivot := GetCellData(theList, first);
			up := first;
			down := last;
			repeat
				while (GetCellData(theList, up) <= pivot) and (up < last) do
					up := up + 1;
				while GetCellData(theList, down) > pivot do
					down := down - 1;
				if up < down then
					ExchangeCellContents(theList, up, down);
			until up >= down;
			ExchangeCellContents(theList, first, down);
			privIndex := down;
		end;
{------}
	begin
		if first < last then
			begin
				if what then
					PartitionGeneric(theList, first, last, privIndex, GetTypeListItem)
				else
					PartitionGeneric(theList, first, last, privIndex, GetRsrcIDListItem);
				QuickSortTypesList(theList, first, privIndex - 1, what);
				QuickSortTypesList(theList, privIndex + 1, last, what);
			end;
	end;

{-----------------------------------------}
{Makes an empty selectable list of all resource types}
	function BuildList (theWindow: WindowPtr; theRect: Rect; columns: Integer; flags: Integer): ListHandle;
		const
			scrollH = False;		{Horizontal scroll bar}
			scrollV = True;		{Vertical Scroll bar}
			hasGrow = False;	{Grow box}
			drawIt = False;		{Display the list}
			LDEF = 0;			{Default text list}
		var
			theList: ListHandle;	{Handle to new list}
			cellSize: Point;			{Size of cell in (h,v) pixels}
			dataBounds: Rect;		{}
	begin
		SetPt(cellSize, (theRect.right - theRect.left) div columns, 11);
		setRect(dataBounds, 0, 0, 0, 0);
		theList := LNew(theRect, dataBounds, cellSize, LDEF, theWindow, drawIt, hasGrow, scrollH, scrollV);
		theList^^.lClikLoop := nil;
		theList^^.SelFlags := flags;
		LongInt(theList^^.userHandle) := columns;	{Store number of columns in LoWord of userhandle}
													{HiWord will come to contain total number of occupied cells}
		BuildList := theList;
	end;

{-----------------------------------------}
{Fill the list with the resource types found in the current resource map}
	procedure FillTypeList (total: Integer; var theList: ListHandle);
		var
			loop: Integer;		{loop for indexing through list}
			rType: ResType;	{current resource type}
			theCell: Point;		{current cell}
			dummy: Integer;	{ignored result from List Manager calls}
			columns: Integer;	{Number of columns in list}
	begin
{The LoWord of the userHandle contains the number of columns in the List}
{The HighWord of the userHandle contains the number of allocated cells in the list}
		columns := LoWord(LongInt(theList^^.userHandle));
		LongInt(theList^^.userHandle) := columns + BSL(total, 16);
		dummy := LAddColumn(columns, MaxInt, theList);
		dummy := LAddRow((total + (columns - 1)) div columns, MaxInt, theList);
		loop := 0;
		while loop < total do
			begin
				loop := loop + 1;
				Get1IndType(rType, loop);
				SetPt(theCell, (loop + 1) mod columns, (loop - 1) div columns);
				LAddToCell(@rType, 4, theCell, theList);
			end;
	end;

{-----------------------------------------}
{Empty the existing list, and update it by filling it with the resource types in the current resource map}
	procedure ChangeTypeList (var theList: ListHandle);
	begin
		LDelColumn(MaxInt, 0, theList);
		LDelRow(Maxint, 0, theList);
		LDoDraw(False, theList);
		FillTypeList(Count1Types, theList);
		if not KeyIsDown(OptionKey) then	{Sort list unless [OPTION] key is held down}
			QuickSortTypesList(theList, 1, GetListArea(theList), True);
		LDoDraw(True, theList);
		LUpdate(theList^^.port^.visRgn, theList);
	end;

{-----------------------------------------}
{Fill out a string with spaces. The Width indicates the maximum width of the string.}
{If how=True then put spaces at end of string, else put spaces at front of string.}
{If the string length > width then chop off the end and put in a '' character.}
	function PadString (how: Boolean; theStr: Str255; width: Integer): Str255;
	begin
		width := Abs(width) mod 256;	{0<=width<=255}
		while Length(theStr) < width do
			if how then
				theStr := ConCat(theStr, ' ')
			else
				theStr := ConCat(' ', theStr);
		if Length(theStr) > width then
			if width > 0 then
				begin
					Delete(theStr, width, 255);
					theStr := ConCat(theStr, '');
				end
			else
				theStr := '';
		PadString := theStr;
	end;

{-----------------------------------------}
{procedure to fill the resource ID list with all occurences of a given type}
	procedure FillRInfoList (rType: ResType; totalCount: Integer; var theList: ListHandle);
		const
			threshHold = 40;	{Ticks to wait before showing the Watch cursor}
		var
			loop: Integer;			{Loop to index through the list of resource ID's}
			rID: Integer;			{ID of resource}
			rName: Str255;		{Name of resource}
			theCell: Point;			{Cell to insert data into}
			tmpStr: Str255;		{Temporary string to be used for string construction}
			theStr: Str255;		{Constructed string to insert into list}
			dummy: Integer;		{Ignored response from List manager calls}
			rsrcHandle: Handle;	{Handle to resoure}
			columns: Integer;		{Number of columns in the list (stored in theList^^.userHandle)}
			timer: LongInt;			{Start time of fill list routine}
			watchHdl: CursHandle;	{Handle to the Watch cursor}
	begin
		timer := TickCount;
		watchHdl := GetCursor(watchCursor);
{The LoWord of the userHandle contains the number of columns in the List}
{The HighWord of the userHandle contains the number of allocated cells in the list}
		columns := LoWord(LongInt(theList^^.userHandle));
		LongInt(theList^^.userHandle) := columns + BSL(totalCount, 16);
		dummy := LAddColumn(columns, MaxInt, theList);
		dummy := LAddRow((totalCount + (columns - 1)) div columns, MaxInt, theList);
		loop := 0;
		SetResLoad(False); 	{Switch off resource loading }
		while loop < totalCount do
			begin
				if TickCount > (timer + threshHold) then	{Is this routine takes too long, show the watch cursor}
					SetCursor(watchHdl^^);
				loop := loop + 1;
				rsrcHandle := Get1IndResource(rType, loop);
				SetPt(theCell, 0, loop - 1);
				GetResInfo(rsrcHandle, rID, rType, rName);
				NumToString(rID, tmpStr);
				theStr := ConCat(PadString(False, tmpStr, 6), '  ');
				if (rType = 'DRVR') & ((Length(rName) > 0) and (rName[1] <> '.')) then	{Driver names have either a leading '.' or NULL character}
					Delete(rName, 1, 1);	{Get rid of the NULL character}
				theStr := ConCat(theStr, PadString(True, rName, 12), ' ');
				if (rsrcHandle^ <> nil) then
					theStr := ConCat('', theStr)		{Resource is currently in memory}
				else if ((rType = 'CODE') and (rID = 0)) then	{Pretend CODE-0 is in memory}
					theStr := ConCat('*', theStr)		{Special prepend char to CODE-0}
				else
					theStr := ConCat(' ', theStr);		{Resource is not in memory}
				NumToString(SizeResource(rsrcHandle), tmpStr);
				theStr := ConCat(theStr, PadString(False, tmpStr, 6));
				LAddToCell(Pointer(LongInt(@theStr) + 1), Length(theStr), theCell, theList);
			end;
		SetResLoad(True); 	{Switch on resource loading }
	end;

{-----------------------------------------}
{Call to fill the given list with all occurences of a given type}
	procedure ChangeRInfoList (rType: ResType; var theList: ListHandle);
	begin
		LDelColumn(MaxInt, 0, theList);
		LDelRow(Maxint, 0, theList);
		LDoDraw(False, theList);
		FillRInfoList(rType, Count1Resources(rType), theList);
		if not KeyIsDown(OptionKey) then	{Sort list unless [OPTION] key is held down}
			QuickSortTypesList(theList, 1, GetListArea(theList), False);
		InitCursor;
		LDoDraw(True, theList);
		LUpdate(theList^^.port^.visRgn, theList);
	end;

{-----------------------------------------}
{Return the number of occupied cells in the given list. This number is stored in the HiWord of the userHandle}
	function GetListArea (theList: ListHandle): Integer;
	begin
		GetListArea := HiWord(LongInt(theList^^.userHandle))
	end;

{-----------------------------------------}
{Goes through the given list starting at the offset as given by 'start', and returns the first}
{selected list item in the 'theCell'.  Also returns the index of theCell in 'start'.   If a selected item}
{was found, the function returns True, else False}
	function GetSelectedItem (var start: Integer; var theCell: Point; theList: ListHandle): Boolean;
		var
			columns: Integer;		{Number of columns in list}
			area: Integer;			{Area (h*v) of list}
			loop: Integer;			{Index for going through the list of cells}
			found: Boolean;			{True=selected cell found, False otherwise}
	begin
		area := GetListArea(theList);
		columns := LoWord(LongInt(theList^^.userHandle));
		loop := start;
		repeat	{Go through list until selected cell found (or end of list reached & none found)}
			loop := loop + 1;
			SetPt(theCell, (loop + 1) mod columns, (loop - 1) div columns);
			found := LGetSelect(False, theCell, theList);
		until found or (loop > area);
		start := loop;
		GetSelectedItem := found;
	end;

{-----------------------------------------}
{Return the currently selected resource type in the type list.}
{If none found then return with an unlikely resource type}
	function GetSelectedRsrcItem (theList: ListHandle): ResType;
		var
			start: Integer;		{Index where to start going through the list}
			theCell: Point;		{The found cell}
{------}
{Return the resource type that is currently selected in the resources type list}
		function ReturnResType: ResType;
			var
				dataLen: Integer;	{Length of ResType (4 bytes)}
				rPtr: Ptr;			{Pointer returned to cell data}
		begin
			dataLen := SizeOf(ResType);
			rPtr := NewPtr(dataLen);
			LGetCell(rPtr, dataLen, theCell, theList);
			ReturnResType := ResTypePtr(rPtr)^;
			DisposPtr(rPtr);
		end;
{------}
	begin
		start := 0;
		if GetSelectedItem(start, theCell, theList) then
			GetSelectedRsrcItem := ReturnResType
		else
			GetSelectedRsrcItem := UnknownRSRC;		{A *Highly* unlikely resource type}
	end;
{-----------------------------------------}

end.