{****************************************************}
{ResourceBits.p																			}
{}
{CodeSucker unit that contains filing & resource related routines}
{}
{Written using Think Pascal v3.0}
{Requires at least System 6 to run}
{}
{ 1991  Mike van Kleef	-	All rights Reserved}
{----------------------------------------------------}
{Address (mail & network) until September 1992 : }
{	42 Melford Road, London,  E11-4PS, England}
{	vankleef@uk.ac.qmc.dcs   [Queen Mary College, London] }
{Address after September 1992 :}
{	Flat 5, 4 St. Quintin Ave., London, W10-6NU, England}
{----------------------------------------------------}
{CodeSucker is distributed in the hope that it will be useful , but WITHOUT ANY WARRANTY}
{Absolutely no-one on Earth accepts responsibility for the consequences of using this program}
{Everyone is hereby granted permission to delete, copy, modify & redistribute CodeSucker}
{}
{Last Change 12/12/91}
{****************************************************}

unit ResourceBits;

interface

	uses
		SysEqu,			{Contains System Globals}
		GlobalRoutines,	{Contains general routines}
		CodeSuckerList;	{List manager things}

	const
{Passed into DoSave to indicate what to save}
		ActionSelSave = 1;	{Save only selected occurences of given Resource Type}
		ActionIDSave = 2;	{Save all occurences of given Resource Type}
		ActionTypeSave = 3;	{Save all resource Types}

	procedure DoSave (srcResFile: Integer; srcFileName: Str255; rType: ResType; action: Integer; theList: ListHandle; doFlags: Boolean);

implementation
{-----------------------------------------}
{Create new output resource file on specified volume.   Returns the file reference number}
	function CreateFile (srcFileName: Str255; var fileRefNum: Integer; var vRefNum: Integer): OSErr;
		var
			where: Point; 			{SFgetfile Dialog POS on screen}
			error: OSErr;			{System error result}
			finderInfo: FInfo;		{Finder Flags}
			writeReply: SFReply;	{Record used by the SFPutfile dialog to return file/volume information}
{------}
{Create output filename using current Application name}
		function FileName: Str255;
			var
				tmpStr: Str255;	{Temporary filename string}
		begin
			tmpStr := srcFileName;		{Name of source resource file}
			Delete(tmpStr, 20, 11);		{Chop off the end in case name is too long}
			FileName := Concat(tmpStr, '.out');	{Add a .out to the end of the filename}
		end;
{------}
	begin
		vRefNum := 0;
		error := fnOpnErr;
		SetPt(where, 80, 200);
		FlushEvents(keyDownMask, 0);	{Flush keyboard input}
		SFPutFile(where, 'Save resources in :', FileName, nil, writeReply);
		if writeReply.good then {File selected}
			begin
				vRefNum := writeReply.vRefNum;
				error := SetVol(nil, writeReply.vRefNum);
				error := GetFInfo(writeReply.fname, writeReply.vRefNum, FinderInfo);
				if error = noErr then
					error := FSDelete(writeReply.fName, writeReply.VRefNum);
				if (error = noErr) | (error = fnfErr) then
					begin
						CreateResFile(writeReply.fName);
						error := ResError;
					end;
				if error = noErr then
					begin
						fileRefNum := OpenResFile(writeReply.fName);
						error := ResError;
					end;
				if error = noErr then
					begin
						finderInfo.fdType := 'rsrc';		{Pretend ResEdit created the file}
						finderInfo.fdCreator := 'RSED';
						error := SetFInfo(writeReply.fname, writeReply.vRefNum, finderInfo);
					end;
			end;
		CreateFile := error;
	end;

{-----------------------------------------}
{Give user an error message with the specified OS error number}
	procedure WarnUser (error: OSErr);
		var
			errorStr: Str255;			{Used for the NumToString conversion of 'error'}
			theWindow: WindowPtr;	{Pointer to the window}
			tmpRect: Rect;				{Rectangle to erase}
	begin
		GetPort(theWindow);
		tmpRect := theWindow^.portRect;
		tmpRect.top := tmpRect.top - 18;
		EraseRect(tmpRect);
		numToString(error, errorStr);
		MoveTo(40, 35);
		ForeColor(redColor);
		DrawString(ConCat('Error occured. ID#', errorStr));
		ForeColor(blackColor);
		MoveTo(40, 60);
		DrawString('<<- Hit rodent to continue ->>');
		SysBeep(0);
		repeat
		until button;
		FlushEvents(mDownMask, 0);	{Clear the mouseEvent to stop it interfering with the controls}
		EraseRect(tmpRect);
		InvalRect(tmpRect);
	end;

{-----------------------------------------}
{A handle is passed in which is the original CODE-0 resource, the jump table at $20(A5)}
{is then copied over the data starting at an offset of 16 bytes from the start, so leaving}
{the original 16 CODE-0 header bytes intact.   The copy of the table is then cleared to an}
{'unloaded' state, and passed back to the calling routine}
{This routine will break some forms of code protection, where CODE is stored}
{on disk in an cyphered form, only to be decyphered once loaded into memory and}
{in the process reading the key from a protected master disk.}
{On entry the current Resource file should be set to the source file}
	procedure Process_CODE0 (var destHdl: Handle);
		const
			headerSize = $10;	{Space taken up by leading Code-0 header bytes}
			JMPinstr = $4EF9;			{JMP}
			MOVEinstr = $3F3C;		{MOVE.W}
			LOADSEGinstr = $A9F0;	{_LoadSeg}
		type
			UnloadedState = record	{JumpTable element in Unloaded state}
					instr1: Integer;		{Routine offset within segment}
					instr2: Integer;		{MOVE.L Segment#,-(A7)}
					instr3: Integer;		{Segment #}
					instr4: Integer;		{_LoadSeg}
				end;
			ElemPtr = ^UnloadedState;
			LoadedState = record	{JumpTable element in loaded state}
					instr1: Integer;		{Segment #}
					instr2: Integer;		{JMP}
					instr3: LongInt;		{Address}
				end;
			ElemPtr2 = ^LoadedState;
		var
			jumpTableAddr: Ptr;		{Address of jump table $20(A5)}
			jumpTableSize: LongInt;	{Size of jumptable}
			index: ElemPtr;				{Unloaded element in jump table}
			index2: ElemPtr2;			{Loaded element in jump table}
			tmpHandle: Handle;			{Handle to CODE-xx resource}
			tmpAddress: LongInt;		{temp address of routine within segment}
			codeNum: Integer;			{ID of CODE-xx resource}
	begin
		jumpTableSize := GetHandleSize(destHdl) - headerSize;
		jumpTableAddr := Ptr(IntPtr(CurJTOffset)^ + LongPtr(CurrentA5)^);
		HLock(destHdl);		{Lock}
{Copy the jump table to the handle, offsetting by the 16 CODE-0 header bytes}
		BlockMove(Ptr(jumpTableAddr), Ptr(LongInt(destHdl^) + 16), jumpTableSize);
		jumpTableAddr := Ptr(LongInt(destHdl^) + 16);	{New address for jump table}
		index := ElemPtr(jumpTableAddr);
		repeat
			if index^.instr2 = JMPinstr then		{Segment currently loaded}
				begin
					Ptr(index2) := Ptr(index);
					tmpAddress := LongInt(LongPtr(index2^.instr3));	{Get routine offset in memory}
					codeNum := index^.instr1;								{Get segment number}
					tmpHandle := GetResource('CODE', codeNum);		{Load segment}
					tmpAddress := tmpAddress - LongInt(tmpHandle^);	{Get routine offset within Segment}
					tmpAddress := tmpAddress - 4;			{Adjust routine offset to the 4 segment header bytes}
					index^.instr1 := LoWord(tmpAddress);	{Reset routine offset in segment}
					index^.instr2 := MOVEinstr;				{Insert MOVE instruction}
					index^.instr3 := codeNum;					{Insert Segment number}
					index^.instr4 := LOADSEGinstr;			{Insert _LoadSeg trap}
				end;
			index := ElemPtr(LongInt(index) + 8);	{Goto next element in table}
		until LongInt(index) > LongInt(jumpTableAddr) + jumpTableSize;
		HUnLock(destHdl);	{Matching Unlock}
	end;

{-----------------------------------------}
{Return the amount of space left on the volume used for the output file}
{Will return 0 if an error occured during the volume information call}
	function SpaceOnVolume: LongInt;
		var
			error: OSErr;			{OS error result}
			volName: Str255;		{Reserve space for volume name}
			vRefNum: Integer;		{Volume reference number}
			freeBytes: LongInt;	{Number of free bytes on the volume}
	begin
		error := GetVol(@volName, vRefNum);
		if error = noErr then
			error := GetVInfo(0, @volName, vRefNum, freeBytes);
		if error = noErr then
			SpaceOnVolume := freeBytes
		else
			SpaceOnVolume := 0;
	end;

{-----------------------------------------}
{Copies specified resource from source file to dest file}
{There appears to be a spurious problem with the copying of resource flags which leads to the }
{copied resources getting garbled resource lengths.   Copying these resources without setting}
{the flags will avoid this problem.}
	function CopyResource (rType: ResType; rID: Integer; srcRsrcFile, destRsrcFile: Integer; copyRsrcFlags: Boolean): OSErr;
		var
			srcHdl: Handle;		{Handle to source resource}
			destHdl: Handle;	{Handle to destination resource}
			error: OSErr;		{OS error result}
			rName: Str255;	{Resource Name}
			attrs: Integer;		{Resource attribute bits}
			inMemory: Boolean;	{True=rsrc is in memory + leave in memory, }
								{False=rsrc wasn't in memory, kick it out again after duplication}
{------}
{Return True if resource currently in memory, else return false}
		function IsRsrcInMemory (rType: ResType; rID: Integer): Boolean;
		begin
			SetResLoad(False); 	{Switch off resource loading }
			IsRsrcInMemory := Get1IndResource(rType, rID)^ <> nil;
			SetResLoad(True); 	{Switch on resource loading }
		end;
{------}
	begin
		error := noErr;
		UseResFile(srcRsrcFile);
		inMemory := IsRsrcInMemory(rType, rID);
		srcHdl := Get1IndResource(rType, rID);
		error := ResError;
		if error = noErr then
			begin
				attrs := GetResAttrs(srcHdl);
				GetResInfo(srcHdl, rID, rType, rName);
				destHdl := srcHdl;
				error := HandToHand(destHdl);
				if error = noErr then
					begin
						if (rID = 0) and (rType = 'CODE') then
							Process_CODE0(destHdl);		{Exception case for CODE-0 Jump Table}
						UseResFile(destRsrcFile);
						if SpaceOnVolume > GetHandleSize(destHdl) then	{Only write if sufficient space on volume}
							if true then
								begin
									AddResource(destHdl, rType, rID, rName);
									if copyRsrcFlags then
										SetResAttrs(destHdl, attrs);	{Only copy resource flags if requested}
									ChangedResource(destHdl);
									WriteResource(destHdl);
									ReleaseResource(destHdl);
								end
							else
								begin
									error := dskFulErr;			{No space on volume to write resource}
									DisposHandle(destHdl);
								end;
					end;
				if not inMemory then
					ReleaseResource(srcHdl);	{Resource wasn't in memory to begin with, so kick it out again}
			end;
		CopyResource := error;
	end;

{-----------------------------------------}
{Write the Selected resource ID's to file   Assumes correct current resource file}
	function WriteSelectedIDs (rType: ResType; srcResFile, destResFile: Integer; theList: ListHandle; doFlags: Boolean): OSErr;
		var
			count: Integer;				{Loops through the resources}
			totalResources: Integer;		{Total number of occurences of given resource}
			dummyCell: Point;			{Unused cell parameter}
			error: OSErr;					{System error result}
	begin
		error := noErr;
		count := 0;
		totalResources := Count1Resources(rType);
		while (error = noErr) and (count < totalResources) do
		{Copy only selected resources of given type to the destination file}
			if GetSelectedItem(count, dummyCell, theList) then
				error := CopyResource(rType, count, srcResFile, destResFile, doFlags);
		WriteSelectedIDs := error;
	end;

{-----------------------------------------}
{Write all resources of given type to file.   Assumes correct current resource file}
	function WriteAllIDs (rType: ResType; srcResFile, destResFile: Integer; doFlags: Boolean): OSErr;
		var
			count: Integer;				{Loops through the resources}
			totalResources: Integer;	{Total number of occurences of given resource}
			error: OSErr;				{System error result}
	begin
		error := noErr;
		count := 0;
		totalResources := Count1Resources(rType);
		while (error = noErr) and (count < totalResources) do
			begin 				{Copy all resources of given type to destination file}
				count := count + 1;
				error := CopyResource(rType, count, srcResFile, destResFile, doFlags);
			end;
		WriteAllIDs := error;
	end;

{-----------------------------------------}
{Save resources of specified type to a new (to be created by the user) file}
{srcResFile:File number of source resource file.}
{srcFileName:File name of source resource file}
{rType:The type of resource which to save}
{selected:True=Save only selected resource ID's, False=save all}
{theList:list containing all occurences of given resource type}
	procedure DoSave (srcResFile: Integer; srcFileName: Str255; rType: ResType; action: Integer; theList: ListHandle; doFlags: Boolean);
		var
			error: OSErr;				{System error result}
			destResFile: Integer;		{Rsrc File reference number of destination file}
			loop: Integer;				{General loop counter}
			totalTypes: Integer;		{total count of specific resource type}
			vRefNum: Integer;			{Volume reference number}
			watchHdl: CursHandle;		{Handle to cursor}
	begin
{copyRsrcFlags := doFlags;}
		error := CreateFile(srcFileName, destResFile, vRefNum);
		if error = noErr then
			begin
				watchHdl := GetCursor(watchCursor);
				SetCursor(watchHdl^^);
				UseResFile(srcResFile);
				case action of
					ActionSelSave: 				{Save only selected occurences of given Resource Type}
						error := WriteSelectedIDs(rType, srcResFile, destResFile, theList, doFlags);
					ActionIDSave: 					{Save all occurences of given Resource Type}
						error := WriteAllIDs(rType, srcResFile, destResFile, doFlags);
					ActionTypeSave: 				{Save all resource Types}
						begin
							totalTypes := Count1Types;
							for loop := 1 to totalTypes do
								begin
									Useresfile(srcResFile);
									Get1IndType(rType, loop);
									error := WriteAllIDs(rType, srcResFile, destResFile, doFlags);
								end;
						end;
					otherwise
				end; {Case}
			end;
		if error <> fnOpnErr then
			begin
				CloseResFile(destResFile);		{Close file, ignoring any error}
				if error <> noErr then	{Drat, we had an error}
					WarnUser(error);
				error := FlushVol(nil, vRefNum); {Flush volume, ignoring any error}
			end;
		InitCursor;	{Restore Arrow cursor}
		UseResFile(srcResFile);	{Reset resource file to what it was}
	end;

end.